;
;
;                       Micro- ADE
;
;                         for the
;
;                           6502
;
;
;                       ASSEMBLER
;
;                       DISASSEMBLER
;
;                       EDITOR
;
;
;
;                           By Peter R.Jennings
;
;
;
;          (C) Copyright, 1977. All Rights reserved.
;
;  Micro-ware Ltd
;
; Original version for KIM-1
;
; Original (C) by Peter Jennings 77.06.28
;
; Retyped from disassembly and source listing in Micro-Ade format
; in assembler syntax standard MOS Technology 
; as close as possible to original source, original comments.
; Checked for being byte exact to listings and dump in kimmicroade.pdf page 51
; Notes
;  - JSR BUFIN is two bytes lower in 'kimmicadeedit.pdf", dump is correct
;  - 25F8 reset symbol table in dump is 90 ORAIM, in source LDY A0(latest is correct)
; ZOUTSP in assembler = OUTSP in IO, correct in dump, 
; 
; Assembled with TASM 32 bit
; tasm -65 -b MICADE.asm MICADE.BIN
;
; Hans Otten, 2021
; Last update August 23 2021
; Status:
;  - base is disassembly of "micro ade printer version 4k", a version very close to original 
;    (adapted to ACIA, VT100 keys, parallel keyboard, H14 printer, core is original)
;  - assembles with TASM, correct size 2000-2FFF, editor, assembler, I/o correct locations
;  - inconsistencies between the three parts resolved in naming labels, one source now
;  - I/O port finished, checked for source and binary equivalent to listing "kimmicadeio.pdf"
;
; *************************
; * MICRO ADE Text Editor * 
; *************************
;
EDITOR   .org    $2000  ;  77.06.28
;
; Constants (added for readability)
CR      = $0D           ; Carriage return
LF      = $0A           ; Linefeed
EOF     = $40           ; end of file symbol
PROMPT  = $2D           ; prompt -
ESC     = $1B           ; escape character
DEL     = $7F           ; delete (replace with something more applicable like 08)
BSL     = $5C           ; backslash \
CTRLE   = $05           ; Control-E
BSP     = $5F           ; backspace (replace with 08)
;
;   PAGE ZERO SYMBOLS
;
BLO     = $10           ; WS pointer
BHI     = $11      
OBLO    = $12           ; WS pointer temp
OBHI    = $13      
LTH     = $14           ; Length of input buffer
NLO     = $15           ; Current Line number
NHI     = $16           ; 
CTR     = $17           ; Number of input parameters
LO      = $18           ; Input temp
HI      = $19         
LOPAR   = $1A          ; Command input parameters
HIPAR   = $1D      
LABEL   = $20           ; Symbol temp label input
OPIN    = $26           ; Instruction input                        
INDX    = $27           ; 
SAVID   = $27           ; ID temporary
SPCNT   = $28           ; Space count for disassembler 
XCLO    = $29           ; Symbol table pointer for disassembler
XCHI    = $2A           ; 
AD      = $29           ; Address mode input
ARGIN   = $2B           ; Argument input
AX      = $2B           ; temporary data
YCOM    = $31           ; Comment pointer
PLAB    = $32           ; packed label 
POP     = $36           ; packed instruction
TMP3C   = $3C           ; Temporary data
PCLO    = $3D           ; Assembly address
PCHI    = $3E      
SLO     = $3F           ; Symbol table pointer
SHI     = $40      
ELO     = $41           ; End of symbol table
EHI     = $42      
SXL     = $43           ; Temp symbol table pointer
SXH     = $44
ADM     = $45           ; Adderss mode
LENGTH  = $46           ; Instruction length
OP      = $47           ; Opcode (object byte 1)
PCALO   = $4A           ; object output pointer
PCAHI   = $4B   
PASS    = $4C           ; pass flag
PRFLAG  = $4D           ; Print flag
SAVE    = $51           ; Save flag
PAGE    = $55           ; Page number
PGN     = $56           ; Line in pagemode
TITLE   = $57           ; Title
XLO     = $55           ; Table temp
XHI     = $56           ;
SAVIDB  = $5D           ; save ID in assembler
CNT     = $54           ; symbol table counter
SALX    = $60           ; START ADDRESS FOR CWRITE
SAHX    = $61      
ID      = $62           ; ID FOR CWRITE
PMODE   = $63           ; Page mode flag
COUNTL  = $64           ; Line count

GANG    = $F0           ; CWRITE pulser
TIC     = $F1           ; CWRITE Timer
COUNT   = $F2           ; CWRITE counter
TMP     = $F3           ; Temporary storage
YTMP    = $F4
XTMP    = $F5
TRIB    = $FE           ; Cycle counter

BUFFER  = $0100         ; I/O BUFFER
;
;   PAGE 17 DATA for KIM CREAD/CWRITE
; 
SAL     = $17F5         ; Tape start address
SAH     = $17F6
EAL     = $17F7         ; Tape end address
EAH     = $17F8
SAVX    = $17E9         ; PACKT temporary
; 
; KIM ROM and PIA addresses
;
SAD     = $1740         ; PIA data port A
PADD    = $1741         ; PIA A data direction
SBD     = $1742         ; PIA location port B data
PBDD    = $1743         ; PIA B data direction
TIMERD  = $1744         ; timer start
TIMER   = $1747         ; timer status
SAD2    = $1700         ; expansion port PIA A data 
PADD2   = $1701         ; expansion port PIA A data direction
SBD2    = $1702         ; expansion port PIA B data
PBDD2   = $1703         ; expansion port PIA B data direction
;
CHKL    = $17E7         ; checksum lo hi
CHKH    = $17E8         ; checksum hi
VEB     = $17EC         ; Volatile Execution Block
INTVEB  = $1932         ; Init VEB subroutine
CHKT    = $194C         ; Checksum routine
INCVEB  = $19EA         ; Increment VEB subroutine
RDBYT   = $19F3         ; Read Byte subroutine
RDCHT   = $1A24         ; Read char routine
RDBIT   = $1A41         ; Read bit subroutine
INIT    = $1E8C         ; Reset PIA ports
;
; KIM vectors
;
IRQVEC  = $17FE         ; KIM-1 IRQ/BRK vector
; 
; Cold start entry for Micro ADE
;
          CLD                   ; clear decimal
          JSR CRLF
;          
          LDA #$FF              ; reset pagemode
          STA PMODE
;
          LDY #$63               ; print New?
          JSR YSNO
          BNE RESTRT            ; If no
          JSR RESB
          LDA #CR               ; format
          JSR STORE             ; workspace
          JSR STORN             ; to CR 00 00 EOF
          BEQ INCCLR
          LDY #$00
CLEAR     LDA #EOF      
          STA (BLO),Y
INCCLR    JSR INCBUF
          LDA BHI
          CMP SOURCF
          BNE CLEAR   
;          
          LDY #$69              ; Print Clear
          JSR MESSY
;
; Warm start entry
;          
RESTRT    JSR CRLF
          LDX #$FF              ; reset Stack and modes
          TXS
          CLD
          CLI
; 
          LDA #(ERROR & $FF)    ; setup IRQ to error routine  
          STA IRQVEC
          LDA #(ERROR >> 8)
          STA IRQVEC + 1
          LDA #$0C              ; turn on cassette PIA
          STA PBDD2
          STA SBD2
;          
          LDA #PROMPT
          JSR OUTCH
          JSR BUFIN
;
; Routine to decode 1 to 3 hexadecimal input parameters
; and save them in LOPAR, HIPAR
;          
          LDA #$00
          LDX #$06
GTCLR     STA HI,X              ; clear old parameters
          DEX
          BNE GTCLR
          STA CTR 
;          
GTA       LDA BUFFER,X
          CMP #'A'              ; skip over command
          BMI REGETA
          INX
          BNE GTA
;
REGETA    DEX
REGET     LDA #$00
          STA LO
          STA HI
          STA SAVX
;          
NXT       INX
          LDA BUFFER,X          ; buffer check  
          CMP #CR               ; for delimiters
          BEQ COMMA
          CMP #' '              
          BEQ NXT
          CMP #','
          BEQ COMMA
;
          JSR PACKT             ; pack
          BNE ERR
          LDA LO                ; four digits into HI,LO
          LDY #$04
ROT       ASL A
          ROL HI
          DEY
          BNE ROT
          ORA SAVX
          STA LO
          STY SAVX
          JMP NXT
;
ERR       BRK                   ; ****<9E> command syntax
;
COMMA     LDY CTR
          PHA
          LDA LO                ; save LOPAR(y)
          STA LOPAR,Y
          LDA HI
          STA HIPAR,Y           ; HIPAR(Y)
;          
          PLA
          INC CTR               ; end of input
          CMP #CR
          BNE REGET
;
          DEC CTR
          BNE NOTON
          LDA LOPAR
          STA LOPAR + 1          ; if only 1 input both same duplicate
          LDA HIPAR
          STA HIPAR + 1
NOTON     LDA BUFFER
          CMP #'A'              ; Add?
          BNE NOTA
;
; *** [Add] ***
;
          JSR FNDND             ; Find EOF
          JSR DECBUF            ; adjust it
          JSR DECBUF
          JSR DECBUF
;          
          LDA #CR               ; change EOF to CR
          LDY #$00
          STA (BLO),Y
;          
          JSR CRLF
NEXTAD    JSR PRNN              ; get input
          JSR STORN
          JSR LINE              ; save N and data
          CMP #EOF
          BEQ ENDAD
          JSR NADJ              ; N = N + 10
          BCC NEXTAD
ENDAD     JSR STORE
          BEQ RSTRTB
;          
NOTA      CMP #'L'
          BNE NOTL
;
; ***[LIST] ***
;
LSFN      JSR LIST
RSTRTB    JMP RESTRT
;
NOTL      CMP #'I'
          BNE NOTI
;
;  *** [Insert] ***
;
          JSR FNDLN             ; Find line N
          SED
          LDA NLO
          SEC
          SBC #$10              ; N = N -10
          STA NLO
          LDA NHI
          SBC #$00
          STA NHI
;
AGN       LDA #$01
          JSR NADJA             ; N = N + 1
          LDA NLO
          CMP LOPAR
          BNE INSRX
          BRK                   ; ****<1C> Insertion overflow
;          
INSRX     JSR PRNN
          LDA NLO
          PHA
          LDA NHI
          PHA
          JSR SPACE             ; create space for new line
          PLA
          STA NHI
          PLA
          STA NLO
;          
          JSR STORN             ; Store new line
          JSR LINE
          CMP #EOF
          BNE AGN
;          
          LDA #' '
          JSR STORE
          LDA #CR
          JSR STORE
          
          LDA NLO
          STA LOPAR             ; delete last line entered
          STA LOPAR + 1
          LDA NHI
          STA HIPAR
          STA HIPAR + 1
          JMP DELFN
;          
NOTI      CMP #'N'
          BNE NOTN
;
;  *** [Number] ***
;          
          JSR RESB
NUMB      JSR LOAD              ; check for end
          BMI ENDN
          CMP #$40
          BEQ ENDN
          CMP #CR
          BNE NUMB              ; Store N, N + 10
          JSR NADJ
          JSR STORN
          JMP NUMB
;           
NOTN      CMP #'D'
          BNE NOTD
;
;  ***[DELETE]***
;          
DELFN     JSR DELETE
ENDN      JMP RESTRT
;
NOTD      CMP #'C'
          BNE NOTC
;
; ***[Clear]***
;
          JMP EDITOR
;          
NOTC      CMP #'F'
          BNE NOTF
;
; ***[Fix]***
;
          JSR LIST
          JSR DELETE
          JMP INSRX
;          
NOTF      CMP #'W'
          BNE NOTW
;
; ***[WHERE]***
; 
          JSR FNDLN             ; Find N
WOUT      LDA BHI
          LDX BLO
          JSR HEXSP             ; print address
          JMP LSFN              ; list N

NOTW      CMP #'S'
          BNE NOTS
;          
          JSR SAVEX
          JMP RESTRT
; 
SAVEX     LDX CTR               ; address given?
          CPX #$02
          BEQ SAVA
          
SAVEY     JSR FNDND
          LDA #$00
          STA LOPAR + 1          ; setup parameters from workspace definition
          LDA SOURCE            
          STA HIPAR + 1
          LDA BLO
          ADC #$06
          STA LOPAR + 2
          LDA BHI
          ADC #$00
          STA HIPAR + 2
;          
SAVA      LDA LOPAR + 1          ; set up save parameters
          STA SAL
          LDA LOPAR + 2
          STA EAL
          LDA HIPAR + 1
          STA SAH
          LDA HIPAR + 2
          STA EAH
          LDA HIPAR + 1
          LDX LOPAR + 1
          JSR HEXSP             ; print addresses
          LDA HIPAR + 2
          LDX LOPAR + 2
          JSR HEXSP
;         
          DEC ID
          LDA LOPAR
          BEQ WRITEA
          STA ID
;          
WRITEA    LDA SAL
          STA SALX              ; call CWRITE
          LDA SAH
          STA SAHX
          JMP WRITE
;
NOTS      CMP #'M'
          BEQ MOVE
          JMP NOTM
;
; ***[MOVE]***
;           
MOVE      LDA LOPAR
          STA LO                ; Find 'TO' line
          LDA HIPAR
          STA HI
          JSR FNDLN
          BNE ERRM
          LDA LOPAR + 1
          STA LOPAR
          LDA HIPAR + 1
          STA HIPAR
;
REMOVEA   LDA BLO
          STA OBLO              ; Save "TO' address
          LDA BHI
          STA OBHI
;
          JSR FNDLN             ; FInd "FROM" line
;
          BEQ OKLN
ERRM      BRK                   ; ******<26> Illegal line
;
OKLN      LDA NLO
          CMP LOPAR             ; Correct line?
          BNE NXTL
          LDA NHI
          CMP HIPAR
          BNE NXTL
;
          JSR INCBUF            ; load buffer
          JSR INCBUF
          JSR BUFOUT
;          
          TXA
          TAY
          DEY
          LDA OBLO              ; create space for line
          STA BLO
          LDA OBHI
          STA BHI
          JSR SPACE
;
          LDA #$00
          JSR STORE             ; Insert line
          JSR STORE
          JSR LINE
;          
NXTL      LDX CTR
          CPX #$02
          BNE DUN
;          
          LDA #$01
          CLC
          SED
          ADC LOPAR             ; reset to search for next line
          STA LOPAR
          LDA HIPAR
          ADC #$00
          STA HIPAR
          CLD
          LDA LOPAR + 2
          CMP LOPAR
          LDA HIPAR + 2
          SBC HIPAR
          BCS REMOVE
;          
DUN       LDX #$FB
SHFT      LDA HIPAR +3,X
          STA HIPAR + 2,X
          INX
          BNE SHFT
          DEC CTR
          BNE TWO
          LDA LOPAR
          STA LOPAR + 1
          LDA HIPAR
          STA HIPAR + 1
TWO       JMP DELFN
;
REMOVE    LDA LOPAR
          PHA
          LDA HIPAR             ; get baclk "TO" line
          PHA
          LDA LO
          STA LOPAR
          LDA HI
          STA HIPAR
          JSR FNDLN
          PLA
          STA HIPAR
          PLA
          STA LOPAR
          JMP REMOVEA           ; start over
;          
NOTM      CMP #'X'
          BNE NOTX
;
; ***[EXECUTE]***
;          
          LDA HIPAR
          ORA LOPAR
          BNE GETADS
          JMP ASMBLR            ; assembler (was label MAIN)
          
GETADS    LDA HIPAR
          STA LOPAR + 1
          JMP (LOPAR)           ; execute
          
NOTX      CMP #'P'
          BNE NOTP
; 
; ***[PAGE MODE]***
;         
          LDA PMODE
          EOR #$FF              ; reset mode and counter
          STA PMODE
          LDA #$F0
          STA COUNTL
          BNE RSTRTA
;
NOTP      CMP #'G'
          BNE NOTG
 ;
; ***[GET FILE]***
;         
          LDA LOPAR
          STA ID
GETRD     JSR READ              ; CREAD
;
          INC ID
          LDA LOPAR + 1          ; more files?
          CMP ID
          BCS GETRD
          BCC RSTRTA
;          
NOTG      CMP #'E'
          BNE NOTE
;
; ***[END]***
;
          JSR FNDND
          LDA BHI
          LDX BLO
          JSR HEXSP             ; print address
          JSR NOUT
RSTRTA    JMP RESTRT
;
NOTE      CMP #'Z'
          BNE NOTZ
;
; ***[DISASSEMBLER]***
;
          LDX CTR
          BNE TWIN              ; (2 pars)
; 
          LDA LOPAR
          ORA HIPAR
          BEQ CONT              ; (0 pars)
;          
TWIN      LDA LOPAR
          STA PCLO              ; set PC
          LDA HIPAR
          STA PCHI
;
CONT      LDA #$F0
          STA LO
TWX       JSR DSMBLR
          LDX CTR               ; end of page?
          BNE CHK
          INC LO
          BMI TWX
          JSR INCH
          CMP #CR
          BNE CONT
          
RSTRTC    BEQ RSTRTA
CHK       LDA PCLO
          CMP LOPAR + 1
          LDA PCHI
          SBC HIPAR + 1
          BCC TWX               ; end address
          BCS RSTRTA
;          
NOTZ      CMP #'T'
          BNE NOTT
;
; ***[TABLE]***
;
          JMP TABLE
;          
NOTT      CMP #'R'
          BNE NOTR
;
; ***[REPRODUCE]***
;
          LDA LOPAR + 1
          STA SAVID
RDTWO     LDA LOPAR
          STA ID                ; ID = LOPAR
          JSR READ              ; copy file
          JSR SAVEY
;
          INC LOPAR
          LDA SAVID             ; last ID
          CMP LOPAR
          BCS RDTWO
          BCC RSTRTA
;          
NOTR      CMP #'B'
          BNE NOTB
;
; ***[BLOCKMOVE]***
;          
          LDY LOPAR + 2          ; rearrange parameters
          LDX LOPAR + 1
          LDA HIPAR
          STX HIPAR
          STA LOPAR + 1
JLP       DEY
          LDA ($1A),Y           ; move block
          STA (HIPAR),Y
          TYA
          BNE JLP
          BEQ RSTRTC
;          
NOTB      BRK                   ; *****<68> command error
;
;        ******************
;     ***EDITOR SUBROUTINES*** 
;        ******************
;
; List 1 or more lines given by LOPAR, HIPAR
;
;
LIST      JSR FNDLN
          LDA HIPAR
          ORA  LOPAR            ; argument?
          BNE PARAMS
;
          LDA #$99
          STA HIPAR + 1
          STA LOPAR + 1
;
PARAMS    JSR LOAD
          STA NHI
          JSR LOAD
          STA NLO
;          
          LDA LOPAR + 1
          CMP NLO               ; all done?
          LDA HIPAR + 1
          SBC NHI
          BCC RETL
;         
          JSR NOUT              ; print N
          LDA #':'
          JSR OUTCH
          JSR OUTSP
          JSR BUFOUT            ; fill buffer
          PHA
          JSR PRBUF             ; print buffer
          PLA
          BPL PARAMS
RETL      RTS
; 
; Delete routine
;
DELETE    JSR FNDLN             ; find end of line
          BPL ADL
          RTS
;          
DLN       LDY #$02
GETY      INY
          LDA (BLO),Y           ; count line length
          CMP #CR
          BEQ CLOSE
          CMP #EOF
          BNE GETY
          RTS
;          
CLOSE     STY LTH
          LDA BLO
          STA OBLO
          LDA BHI
          STA OBHI
;          
LOOP      LDY $14
          LDA (BLO),Y           ; close line to EOF
          LDY #$00
          STA (BLO),Y
          JSR INCBUF
          LDA $SOURCF
          CMP BHI
          BNE LOOP
;
          LDA OBLO
          STA BLO
          LDA OBHI
          STA BHI
;          
ADL       LDY #$02
          LDA LOPAR + 1
          CMP (BLO),Y
          DEY
          LDA HIPAR + 1
          SBC (BLO),Y
          BCS DLN
          RTS
;
; Decrement pointer BLO, BHI
;          
DECBUF    LDA BLO
          SEC
          SBC #$01
          STA BLO
          LDA BHI
          SBC #$00
          STA BHI
          RTS
;
; Reset pointer to start of workspace
;          
RESB      LDA #$FF
          STA BLO
          LDA SOURCM
          STA BHI
          LDA #$00
          STA NLO
          STA NHI
          RTS
;
; Print error message
;          
ERROR     PLA                   ; Get return address
          PLA
          JSR ERRPRT
          JMP RESTRT
 ;         
ERRPRT    PHA
          JSR CRLF
          LDY #LF
STAR      LDA #'*'              ; Print ******<
          JSR OUTCH
          DEY
          BPL STAR
          LDA #'<'
          JSR OUTCH
          PLA
          JSR HEXOUT
          LDA #'>'
          JMP OUTCH
;
; Store NLO, NHI
;          
STORN     LDA NHI
          JSR STORE
          LDA NLO
;
; Store one byte, check for overflow
;
STORE     JSR INCBUF
          LDY SOURCF
          CPY BHI
          BNE OKS
          BRK                   ; ****<3B> Source overflow
;          
OKS       LDY #$00
          STA (BLO),Y
          RTS
;          
; N=N+ 10    
;      
NADJ      LDA #$10
NADJA     CLC
          SED
          ADC NLO               ; 16 bit decimal add
          STA NLO
          LDA NHI
          ADC #$00
          STA NHI
          CLD
          RTS
;
; Find line N
;          
FNDLN     JSR RESB              ; reset pointer
;
LODE      JSR LOAD              ; look for CR
          BMI ENDF
          CMP #CR
          BNE LODE
 ;         
          JSR LOAD
          STA NHI               ; Get N
          JSR LOAD
          STA NLO
          CMP  LOPAR            ; compare LOPAR, HIPAR
          LDA NHI
          SBC HIPAR
          BCC LODE
ENDO      JSR DECBUF            ; adjust BLO, BHI
          JSR DECBUF
          LDA #$00
ENDF      RTS
;
; Store buffer in workspace
;
LINE      LDX #$00
STR       LDA BUFFER,X
          CMP #$40
          BEQ ENDLN
          JSR STORE
          INX
          CMP #CR               ; end of line
          BNE STR
ENDLN     RTS
;
; Prompt "N:" and input to buffer
;
PRNN      JSR NOUT
          LDA #':'
          JSR OUTCH
          JSR OUTSP
          JMP BUFIN
;
; Find EOF record
;          
FNDND     JSR RESB
LD        JSR LOAD              ; look for first EOF character
          CMP #EOF
          BNE NOTEN
          RTS
;          
NOTEN     CMP #CR
          BNE LD
          JSR LOAD
          STA NHI
          JSR LOAD              ; Get N
          STA NLO
          JMP LD
;   
; Create a space Y+3 characters long from BLO, BHI
;       
SPACE     INY
          INY
          INY
          STY LTH
          LDA BLO               ; save BLO, BHI
          STA OBLO
          LDA BHI
          STA OBHI
          JSR FNDND
          
SPACEIT   LDY #$00
          LDA (BLO),Y
          LDY $14
          STA (BLO),Y
          JSR DECBUF
          CMP SOURCM
          BNE NOERS
          BRK                   ; *****<D4> Moving below source!
          
NOERS     CMP OBHI
          BNE SPACEIT
          LDA BLO
          CMP OBLO
          BNE SPACEIT
          RTS
;
; Fill buffer with current line
;
BUFOUT    LDX #$00
LDX       JSR LOAD
          STA BUFFER,X
          INX
          CPX #$40              ; limit of 64 characters
          BPL OVFLOW
;          
          CMP #CR
          BEQ ENDOT
          CMP #EOF
          BNE LDX
;          
OVFLOW    LDA #CR
          STA BUFFER,X          ; ret n=1 of overflow
          LDA #$FF
ENDOT     RTS
;
; Load from workspace checking for EOF
;
LOAD      JSR INCBUF
          LDY $SOURCF
          CPY BHI
          BNE NOEND
          LDA #$FF              ; ret n=1 if EOF reached
          RTS
;
NOEND     LDY #$00
          LDA (BLO),Y
          RTS
;
; Increment BLO, BHI
;
          
INCBUF    INC BLO
          BNE RETI
          INC BHI
RETI     RTS
;
; Symbol table routine
;
TABLE     LDY #$4B              ; print "Symbol table"
          JSR MESSY
          LDA SYMBOL
          LDX #$00
          JSR HEXSP
          LDA EHI               ; start and end addresses
          LDX ELO
          JSR HEXAX
;
          LDX  LOPAR
          CPX #$02
          BEQ LONLY             ; (T2 command)
;
          BMI SORT
;          
          LDA LOPAR + 1
          STA ELO               ; set ELO, EHI
          LDA HIPAR + 1
          STA EHI
          BNE LONLY
;
; bubble sort routine
;
SORT      JSR SETSY
LOOPT     LDX  LOPAR
          BNE LOOPB
          LDY #$00
NEXTY     LDA (XLO),Y           ; compare for alpha sort
          CMP (SLO)),Y
          BCC MOVX
          BNE LESS
          INY
          CPY #$04
          BNE NEXTY
          BEQ LESS
;
LOOPB     LDY #$04
          LDA (XLO),Y
          CMP (SLO),Y           ; prepare for address sort
          INY
          LDA (XLO),Y
          SBC (SLO),Y
          BCS LESS
          
MOVX      LDY #$05
MOVIT     LDA (SLO),Y           ; exchange symbols
          PHA
          LDA ($XLO),Y
          STA (SLO),Y
          PLA
          STA (XLO),Y
          DEY
          BPL MOVIT
          INC CNT
;          
LESS      JSR DADS              ; S=S+6
          LDX #$16
          JSR DADX              ; X=X+6
          BCC LOOPT
;          
          LDX CNT               ; Finished?
          BNE SORT
;
          LDA #$00
          JSR SETSY
PRNT      INC CNT
          BMI PRNTX             ; print new line
          JSR CRLF
          LDA #$FC
          STA CNT
;          
PRNTX     JSR PRSYM             ; print symbol and address
          LDY #$04
          LDA (SLO),Y
          TAX
          INY
          LDA (SLO),Y
          JSR HEXAX
          JSR DADS
          BCC PRNT
;
LONLY     JMP RESTRT
;
; Routine to print 4 spaces and symbol address
;
PRSYM     LDY #$04
PRX       JSR OUTSP
          DEY
          BNE PRX
;       
; Routine to print symbol
;   
DPSYM     LDY #$00
LOD       LDA (SLO),Y
          STA LABEL,Y
          INY
          CPY #$04              ; unpack 4 bytes into 5 bits ASCII
          BNE LOD               ; for 6 char symbol
          LDY #$06
          LDX #$06
A         LDA #$00
ROTT      ASL LABEL +3
          ROL LABEL + 2
          ROL LABEL + 1
          ROL LABEL
          ROL A
          AND #HIPAR + 2
          DEX
          BNE ROTT
          CLC
          ADC #$40
          CMP #$40              ; 40 = space
          BNE NOSP
          LDA #$20
NOSP      JSR OUTCH
          LDX #$05
          CPY #$04              ; fill blank ts to end of characters
          BNE X
          LDX #$06
X         DEY
          BNE A
          JMP OUTSP
;
; reset SLO. XLO
;          
SETSY     LDA #$00
          STA SLO
          STA CNT
          LDA #$06
          STA XLO
          LDA SYMBOL
          STA SHI
          STA XHI
          RTS
;    
; Assembler  77.06.28   
; ***********************
; * MICRO-ADE Assembler *
; * (C) COPYRIGHT 1977  *
; * MICRO-WARE LIMITED  *
; * CANADA              *
; ***********************
          
ASMBLR    LDY #00               ; reset symbol table
          STY ELO
          LDA SYMBOL
          STA EHI 
;
; Entry without Symbol table reset
;          
          JSR MESSY
          LDA #$FF              ; set Pass 1
          STA PASS
IDAS      LDY #$5E
GETID     JSR MESSY             ; Print ID         
          INC LOPAR + 2
          LDA HIPAR + 2          ; Auto ID
          CMP LOPAR + 2
;
          BCC GETIT          
          LDA LOPAR + 2
          STA ID
          JSR HEXOUT            ; Print ID
          JSR CRLFA
          JMP NOERID
;          
GETIT     JSR BUFIN
          LDA BUFFER            ; input
          CMP #CR
          BNE NENDID
          JMP ENDID
;          
NENDID    LDX BUFFER + 1
          JSR PACKX             ; pack ID
          BNE ERID1
          STA LOPAR + 2
          STA ID
          LDA BUFFER + 2
          CMP #','
          BNE NOERID            ; more than 1
          LDA BUFFER + 3
          LDX BUFFER + 4
          JSR PACKX             ; PAack ID 2
          STA HIPAR + 2
          BEQ NOERID
;          
ERID1     LDY #$67             ; print "?"
          BNE GETID
          ; 
NOERID    LDA ID
          BEQ OKRD
          JSR READ              ; read file
;          
OKRD      JSR RESB              ; reset BLO
          JSR INCBUF
          LDA #$FF
          STA PCALO             ; object output reset
          LDA OBJECT            
          SEC
          SBC #$01
          STA PCAHI
;
; Entry to continue from break
;  
ERRTRY    LDX #$FF              ; Reset stack and IRQ
          CLI
          TXS
          LDA #(ERRORA & $FF)
          STA IRQVEC
          LDA #(ERRORA >> 8)
          STA IRQVEC + 1
;          
          JSR LOAD              ; Get N of line
          STA NHI
          JSR LOAD
          STA NLO
          JSR BUFOUT            ; Put line on buffer 
;
          BMI ASMBLD
          JSR INTRP             ; interpret input syntax
;
          JSR DECODE            ; decode op
          STA OP
          JSR ADJLTH
;          
          JSR SPECOP            ; check for pseudo-op
          LDX PASS
          BNE ONEX
          JSR GETARG            ; interpret argument
;          
          JSR RELAD             ; calculate relative addresses
; 
          JSR PRINT             ; print line
          BEQ DNX
;
ONEX      JSR ADDSYM            ; add symbol to table
;
DNX       JSR PCADJ             ; adjust PC

          JMP ERRTRY
; 
ASMBLD    LDX PASS              ; pass 2 = 0
          BNE IDXS
;      
          LDX SAVE
          BNE IDXS
          INC SAVIDB            ; Save object code
          LDA SAVIDB
          STA ID
          STX SAL
          LDA OBJECT
          STA SAH
          LDA  PCALO
          ADC #$01
          STA EAL
          LDA PCAHI
          ADC #$00
          STA EAH
          JSR WRITE
;         
          LDA PCLO              ; reset object parameters
          STA SALX
          LDA PCHI
          STA SAHX
;
IDXS      JMP IDAS              ; get next tape
;
ENDID     LDA PASS
          BEQ ENDB
;
; Entry for Pass 2
;          
          LDA #$00              ; pass = 2 (00)
          STA PASS
;
          LDA #$00
          STA PAGE
          STA PGN
;
          LDY #$08              ; print "PRINT:"
          JSR YSNO
          BNE NOPRT             ; set print flag
          LDA #$00
NOPRT     STA PRFLAG
;
ERISD     LDY #$18              ; print "SAVE ID:"
          JSR MESSY
          JSR BUFIN             ; input ID
          LDA BUFFER
          CMP #CR
          BEQ NVSE
          LDX BUFFER + 1
          JSR PACKX             ; pack ID
          BNE ERISD
          STA SAVIDB
          DEC SAVIDB
          LDA #$00              ; set save flag
NVSE      STA SAVE
          JMP IDAS              ; assemble
;
ENDB      JMP RESTRT            ; return to editor
;
;     *********************
; *** ASSEMBLER SUBROUTINES***
;     *********************
; Pseudo operation decode
; 

SPECOP    LDA OP
          CMP #$0C              ; 'ORG'
          BNE NTPC
       
          LDX #$05
MVTIT     LDA LABEL,X           ; setup title
          STA TITLE,X
          DEX
          BPL MVTIT
;          
          JSR GETARG            ; decode argument
;
          LDA OP + 2            ; setup PC and save for assembly
          STA SAHX
          STA PCHI
          LDA OP + 1
          STA SALX
          STA PCLO
;          
LTHX      LDA #$FF              ; length = 0
          STA LENGTH
          RTS
;
NTPC      CMP #$80              ; comment line
          BEQ LTHX
;
          CMP #$FA              ; * operation
          BEQ LTHX
;          
          CMP #$4C              ; skip some sepecila ops
          BEQ RTO
          CMP #$6C
          BEQ RTO
;
          AND #$0C              ; optimizable codes?
          CMP #$0C
          BNE RTO
;
          LDA ARGIN             ; check for $XXXX args       
          CMP #$24
          BNE NOTDOL
;
          LDA ARGIN + 1
          ORA ARGIN + 2
          CMP #$30
          BEQ REDUCE
          BNE RTO
;
NOTDOL    LDA #$38              ; lookup symbolic argument
          STA  SXL
          JSR FNDSBL
          BNE RTO
;
          INY
          LDA (SLO),Y           ; zero-page?
          BNE RTO
;
REDUCE    LDA OP                
          AND #$F7              ; optimize
          STA OP
          DEC LENGTH
RTO       RTS
;
; Routine to print hex byte
;
HEXR      LDX PRFLAG
          BNE RTO
HEXX      JMP HEXOUT
;
; print routines with print flag
;
CRLFA     LDA #CR               ; CR and LF
          BNE OUTCHA
;
OUTSPA    LDA #' '              ; space
;
OUTCHA    LDX PRFLAG
          BNE RTO
;
OUTX      JMP OUTCH
;
; print messg(Y) and check for "YES" response 
;
YSNO      JSR MESSY
          JSR BUFIN
          LDA BUFFER
          CMP #'Y'
          RTS
;
; print messg(Y)
;
;
MESSY     LDA MESSG,Y
          INY
          CMP #EOF
          BEQ ENDM
          JSR OUTX
          BNE MESSY
ENDM      RTS
;
; Get one word from buffer
;
GET     LDA BUFFER,Y
          CMP #CR               ; end?
          BNE ST
          DEY
          LDA #' '              ; fill blanks
ST        STA LABEL,X
          INX
          INY
          CMP #' '              ; copy to delimiter (space)
          BNE GET
          RTS
;
; 16 bit Add
;
DADS      LDX #$00
DADX      LDA #$06
DADA      CLC
          ADC SLO,X
          STA SLO,X
          LDA SHI,X
          ADC #$00
          STA $SHI,X
          LDA SLO,X
          CMP ELO
          LDA SHI,X
          SBC EHI
          RTS
;
; Interpret input syntax
;
INTRP     LDX #$1C
          LDA #' '              ; clear input temporaries
LOOPC     STA LABEL,X
          DEX
          BPL LOOPC
;
          LDY #$00
          INX                   ; get label
          JSR GET
;
          LDX #$06
          JSR GET               ; get OP/AD
          CPX #$0A
          BNE NOT8
          STA LABEL,X
;
NOT8      LDX #$0B              ; get argument comment pointer
          JSR GET
          STY YCOM
;
          LDY #$00
          LDX #$00
          JSR PACKYX
          LDY #$03
          LDX #$02
          JSR PACKYX
          LDY #$06
          LDX #$04
          JSR PACKYX            ; pack op
          LDY #$0B
          LDX #$06
          JSR PACKYX            ; pack arg
          LDY #$0E
          LDX #$08
;
; routine to pack 3 chars in 2 bytes
;
PACKYX    LDA LABEL,Y           ; In(y, Y=1, Y+2)
          STA PLAB,X            ; Out (X, X+1)
          LDA LABEL + 2,Y
          AND #$1F
          STA TMP3C
          LDA LABEL + 1,Y
          ASL A
          ASL A                 ; 0AAAAABB
          ASL A                 ; BBBCCCCC
          ASL A
          ROL PLAB,X
          ROL A
          ROL PLAB,X
          ORA TMP3C
          STA PLAB + 1,X
          RTS
;
; Lookup symbol in table
;          
FNDSBL    LDA #$FA
          STA SLO               ; reset SLO, SHI
          LDA SYMBOL
          SEC
          SBC #$01
          STA SHI
;
EACH      LDY #$00
CHECK     LDA (SLO),Y           ; look for match
          CMP (SXL),Y
          BNE NOMTCH
          INY
          CPY #$04
          BNE CHECK
          RTS                   ; symbol found
;          
NOMTCH    JSR DADS
          BCC EACH
          LDA #$FF
          RTS                   ; symbol not found
;
; Add symbol to table
;          
ADDSYM    LDA LABEL
          CMP #' '              ; skip blank
          BNE NOTBL
          RTS
;          
NOTBL     LDA #$32
          STA SXL
          LDA #$00              ; check for duplicate
          STA SXH
          JSR FNDSBL
          BNE OK
          BRK                   ; ***<6F> Duplicate symbol
;
OK        LDY #$00
LODA      LDA (SXL),Y           ; add to table
          STA (ELO),Y
          INY
          CPY #$04
          BNE LODA
;
          LDX OP                ; * operation
          CPX #$FA
          BNE ORDAD
          JSR GETARG
          LDA OP + 1            ; use argument
          LDY #$04
          STA (ELO),Y
          LDA OP + 2
          INY
          BNE ADTABL
;
ORDAD     LDA PCLO
          STA (ELO),Y
          INY
          LDA PCHI
ADTABL    STA (ELO),Y
          LDX #$02              ; adjust ELO, EHI
          JSR DADX
          LDA EHI
          CMP ENDSYM
          BNE RETA
          BRK                   ; ***<A4> Table overflow
;
RETA      RTS
;
; Error routine
;
ERRORA    PLA
          PLA
          JSR ERRPRT            ; print ***<#>
          JSR NOUT              ; print line n#
          LDA #$00
          STA OP                ; clear object code
          STA OP + 1
          STA OP + 2
          LDA #$02
          STA LENGTH
          LDA PRFLAG
          PHA
          LDA #$00              ; print line
          STA PRFLAG
          LDX PASS
          BNE PASONE
          JSR PRINT
          BEQ PASTWO
PASONE    JSR PRBUFA
PASTWO    PLA
          STA PRFLAG
          JSR PCADJ
          JMP ERRTRY            ; continue assembly
;
; decode op-code/address mode 
;          
DECODE    LDX #$09
NEXT      LDA AD
          CMP TABLL,X
          BEQ LEFT
NEXTX     DEX
          BPL NEXT
          BRK                   ; ***<E2> ADDRES MODE NOT FOUND
;
LEFT      LDA AD + 1
          CMP TABLR,X
          BNE NEXTX
          STX ADM               ; ADM = index to TABLL,R
; 
          CPX #$00
          BNE NOTIM
          LDY #$37              ; in limit
          BNE YIN
;
NOTIM     LDY #$3F
YIN       LDA POP               ; get instruction
          CMP MNL,Y
          BNE NEXTYD            ; get instruction code from MNL, MNR
          LDA POP + 1
          CMP MNR,Y
          BEQ HERE
NEXTYD     DEY
          BPL YIN
          BRK                   ; ***<07> unrec instruction        
HERE      TYA
          CMP #$35              ; check for errors
          BNE ORDIN
          CPX #$06
          BEQ VERIF
;
ORDIN     CMP #PCLO
          BNE ORDB
          CPX #$07
          BEQ VERIF             ; (LDXY)
;
ORDB      CMP LOM,X
          BCC ERRI
          CMP HIM,X
          BCC VERIF
ERRI      BRK                   ; ***<23> illegal address mode
;
VERIF     CPY #$30
          BPL TTWO
          LDA FMTA,X            ; decode oplength
          BNE XX
TTWO      LDA FMTB,X
XX        STA TMP3C
          AND #$07
          STA LENGTH
;
          TYA
          LDX #$03
ROTA      ROL TMP3C
          ROL A
          DEX                   ; encode opcode
          BNE ROTA
          BCC OUTA
          ROL TMP3C
          ROL A
          BCS SETB
          BPL OUTB
SETB      ROL TMP3C
          ROL A
          BCC OKC
          AND #$FD
OKC       RTS
OUTA      AND #$F8
          RTS
OUTB      AND #$70
          ORA #$8A
          RTS
;
; routine to fix odd lengths
;          
ADJLTH    LDX LENGTH
          CPX #$03              ; (6 is dummy length)
          BMI NOT6
          LDA OP
          CMP #$20              ; JSR
          BEQ ABS
          AND #$1F
          CMP #$10
          BNE NTREL             ; relative op
          LDX #$01
          BNE STOREA
NTREL     AND #$0F
          CMP #$0B              ; >XB
          BPL ABS
          LDX #$00
          BEQ STOREA
ABS       LDX #$02
STOREA    STX LENGTH
NOT6      RTS
;
; Adjust PC 
;
PCADJ     LDA LENGTH
          BMI RETPCA
          SEC                   ; PC = PC + length + 1
          ADC PCLO
          STA PCLO
          LDA PCHI
          ADC #$00
          STA PCHI
RETPCA    RTS
;
; get hex value for argument
;
GETARG    LDA ARGIN
          CMP #' '
          BEQ RETPCA            ; No arg
          CMP #'$'
          BEQ GHEX              ; hex arg
          CMP #'''
          BEQ ASCI              ; ascii arg
          LDA #$38
          STA  SXL              ; symbolic arg
          LDA #$00
          STA $44
          JSR FNDSBL            ; look it up
          BEQ OKSA
          BRK                   ; ***<A8> undefined symbol 
;
OKSA      LDA (SLO),Y
          STA OP + 1
          INY
          LDA (SLO),Y           ; symbol value
          STA OP + 2
;
          LDY YCOM
          LDA BUFFER,Y
          CMP #'/'
          BEQ ONE
          CMP #'+'
          BEQ POFF
          CMP #'-'
          BNE RETPCA
          DEC OP + 2
POFF      AND #$02
          PHA
          INY
          LDA BUFFER,Y          ; get offset
          INY
          LDX BUFFER,Y
          JSR PACKX
          PLP
          BEQ PLUSO
          EOR #$FF
          SEC
PLUSO     LDX #$09
          JMP DADA + 1
ASCI      LDA ARGIN + 1         ; 16 bit calculation
          STA OP + 2            ; ASCII data
          BNE ONE
;
GHEX      LDA ARGIN + 1         ; hex data
          LDX ARGIN + 2
          JSR PACKX
          STA OP + 2
          LDA ARGIN + 3
          LDX ARGIN + 4
          JSR PACKX
          BEQ TWOA
ONE       LDA OP + 2
TWOA      STA OP + 1
          RTS
;
; routine to print listing
;
PRINT     INC PGN
          BMI OKPAG
          LDA #$0C              ; new page
          JSR OUTCHA
          JSR CRLFA
;
          LDY #$00
TIT       LDA TITLE,Y           ; title
          JSR OUTCHA
          INY
          CPY #$06
          BNE TIT
;
          LDY #$23
MESSPR    LDA MESSG,Y           ; header
          CMP #$40
          BEQ DUMMES
          JSR OUTCHA
          INY
          BNE MESSPR
;
DUMMES    LDA PAGE              ; page number
          SED
          CLC
          ADC #$01
          STA PAGE
          CLD
          JSR HEXR
          LDY #$03              ; skip 4 lines
CRA       JSR CRLFA
          DEY
          BNE CRA
          LDA #$C8
          STA PGN
;
OKPAG     JSR CRLFA
          LDA NHI
          JSR HEXR              ; print line number
          LDA NLO
          JSR HEXR
          LDA #$3A
          JSR OUTCHA
          JSR OUTSPA
;
          LDA OPIN
          CMP #' '
          BNE NOCOMN
;
          LDY #$0E
OUTS      JSR OUTSPA            ; comment line
          DEY
          BPL OUTS
          BMI COMNT
;
NOCOMN    LDA PCHI
          JSR HEXR              ; PC
          LDA PCLO
          JSR HEXR
          JSR OUTSPA
;
          LDA OP
          LDX LENGTH            ; object
          JSR OUTSTR
          LDA OP + 1
          LDX LENGTH
          DEX
          JSR OUTSTR
          LDA OP + 2
          LDX LENGTH
          DEX
          DEX
          JSR OUTSTR
;
PRBUFA    JSR OUTSPA
          LDY #$00
OUTP      CPY #$06
          BEQ SPCE
          CPY #$0B
          BNE NOSPA
SPCE      JSR OUTSPA            ; format label, op & arg
NOSPA     LDA LABEL,Y
          JSR OUTCHA
          INY
          CPY #$11
          BNE OUTP
          JSR OUTSPA
COMNT     LDY YCOM              ; comment
; 
LDY       LDA BUFFER,Y
          CMP #CR
          BEQ RETX
          JSR OUTCHA
          INY
          BNE LDY
RETX      RTS
;
; routine to print objectcode or space
;
OUTSTR    BMI DONE
          LDY #$00
          INC PCALO
          BNE XOT
          INC PCAHI
XOT       STA (PCALO),Y
          JSR HEXR
          JMP OUTSPA
DONE      JSR OUTSPA
          JSR OUTSPA
          JMP OUTSPA
;
; relative branch calculation
;
RELAD     LDA OP
          CMP #$DA
          BEQ EQU               ; = pseudo op
          AND #$1F
          CMP #$10              ; rel op
          BNE RETREL
;
          LDA OP + 1
          SEC
          SBC #$02              ; calculate offset
          PHA
          LDA OP + 2
          SBC #$00
          STA OP + 2
          PLA
          SEC
          SBC PCLO
          STA OP + 1
          LDA OP + 2
          SBC PCHI
          LDA OP + 1
          BCS PLUS
          BMI RETREL
;          
EREL      BRK                   ; ***<F6> range error
;
PLUS      BMI EREL
RETREL    RTS
EQU       LDA OP + 2            ; = pseaudo op
          STA OP
          RTS
;
; Pasck ascii to hex
;
PACKX     JSR PACKT             ; input in A & X
          BNE ERP
          TXA
          JSR PACKT
ERP       RTS
;
; Table for assembler
;
; Left hand mnemonic
;
MNL 
          .byte $0A     ; BRK
          .byte $41     ; PHP
          .byte $0A     ; BPL
          .byte $0D     ; CLC
          .byte $2A     ; JSR      
          .byte $41     ; PLP
          .byte $09     ; BMI
          .byte $4C     ; SEC
          .byte $4A     ; RTI
          .byte $41     ; PHA
          .byte $0A     ; BVC
          .byte $0D     ; CLI
          .byte $4A     ; RTS
          .byte $41     ; PLA
          .byte $0A     ; BVS
          .byte $4C     ; SEI
          .byte $80
          .byte $10     ; DEY
          .byte $08     ; BCC
          .byte $53     ; TYA
          .byte $30     ; LDY
          .byte $50     ; TAY
          .byte $08     ; BCC
          .byte $0D     ; CLV
          .byte $0E     ; CPY
          .byte $25     ; INY
          .byte $09     ; BNE
          .byte $0D     ; CLD
          .byte $0E     ; CPX
          .byte $25     ; INY
          .byte $08     ; BEQ
          .byte $4C     ; SED
          .byte $53     ; TXA
          .byte $53     ; TXS
          .byte $50     ; TAX
          .byte $52     ; TSX
          .byte $10     ; DEX
          .byte $F4     ; *
          .byte $39     ; NOP
          .byte $A8     ; =
          .byte $3E     ; ORA
          .byte $05     ; AND
          .byte $15     ; EOR
          .byte $04     ; ADC
          .byte $4E     ; STA
          .byte $30     ; LDA
          .byte $0D     ; CMP
          .byte $4C     ; SBC
          .byte $06     ; ASL
          .byte $49     ; ROL
          .byte $32     ; LSR
          .byte $49     ; ROR
          .byte $4E     ; STX
          .byte $30     ; LDX
          .byte $10     ; DEC
          .byte $25     ; INC
          .byte $3E     ; ORG
          .byte $09     ; BIT
          .byte $29     ; JMP
          .byte $29     ; JMI
          .byte $4E     ; STY
          .byte $30     ; LDY
          .byte $0E     ; CPY
          .byte $0E     ; CPX
;
; Right half mnemonic
;  
MNR 
          .byte $4B     ; BRK
          .byte $10     ; PHP
          .byte $0C     ; BPL
          .byte $83     ; CLC
          .byte $72     ; JSR
          .byte $90     ; PLP
          .byte $A9     ; BMI
          .byte $A3     ; SEC
          .byte $89     ; RTI
          .byte $01     ; PHA
          .byte $C3     ; BVC
          .byte $89     ; CLI
          .byte $93     ; RTS
          .byte $81     ; PLA
          .byte $D3     ; BVS
          .byte $A9     ; SEI
          .byte $00          
          .byte $B9     ; DEY
          .byte $63     ; BCC
          .byte $21     ; TYA
          .byte $99     ; LDY
          .byte $39     ; TAY
          .byte $73     ; BCC
          .byte $96     ; CLV
          .byte $19     ; CPY
          .byte $D9     ; INY
          .byte $C5     ; BNE
          .byte $84     ; CLD
          .byte $18     ; CPX
          .byte $D8     ; INY
          .byte $B1     ; BEQ
          .byte $A4     ; SED
          .byte $01     ; TXA
          .byte $13     ; TXS
          .byte $38     ; TAX
          .byte $78     ; TSX
          .byte $B8     ; DEX
          .byte $00     ; *  
          .byte $F0     ; NOP
          .byte $00     ; =  
          .byte $41     ; ORA
          .byte $C4     ; AND
          .byte $F2     ; EOR
          .byte $83     ; ADC
          .byte $81     ; STA
          .byte $81     ; LDA
          .byte $B0     ; CMP
          .byte $43     ; SBC
          .byte $6C     ; ASL
          .byte $EC     ; ROL
          .byte $72     ; LSR
          .byte $F2     ; ROR
          .byte $98     ; STX
          .byte $98     ; LDX
          .byte $A3     ; DEC
          .byte $C3     ; INC
          .byte $47     ; ORG
          .byte $34     ; BIT
          .byte $B0     ; JMP
          .byte $A9     ; JMI
          .byte $99     ; STY
          .byte $99     ; LDY
          .byte $19     ; CPY
          .byte $18     ; CPX  
;
; Mode formats
;
FMTA
          .byte $49
          .byte $29
          .byte $48
          .byte $6E
          .byte $89
          .byte $A9
          .byte $CA
          .byte $EA
          .byte $A9
          .byte $09
;
; FORMAT B
;
FMTB
          .byte $11
          .byte $31
          .byte $50
          .byte $76
          .byte $91
          .byte $B1
          .byte $F2
          .byte $F2
          .byte $B1
          .byte $11
;
; Left Address mode
;
TABLL
          .byte $49     ; IM
          .byte $5A     ; Z
          .byte $41     ; A
          .byte $20
          .byte $49     ; IY
          .byte $5A     ; ZX
          .byte $41     ; AY
          .byte $41     ; AX
          .byte $5A     ; ZY
          .byte $49     ; IX
;
; Right addresss mode
;
TABLR
          .byte $4D
          .byte $20
          .byte $20
          .byte $20
          .byte $59
          .byte $58
          .byte $59
          .byte $58
          .byte $59
          .byte $58
;
; Low range
;
LOM
          .byte $14
          .byte $28
          .byte $30
          .byte $00
          .byte $28
          .byte $28
          .byte $28
          .byte $28
          .byte $34
          .byte $28
; Hi range
HIM
          .byte $36
          .byte $40
          .byte $34
          .byte $40
          .byte $30
          .byte $3E
          .byte $30
          .byte $39
          .byte $36
          .byte $30
;
; Disassembler special table
;
TABL
          .byte $38
          .byte $30
          .byte $28
;
; DSMBL Mode table
;
MODE
          .byte $40
          .byte $02
          .byte $43
          .byte $04
          .byte $40
          .byte $06
          .byte $40
          .byte $08
          .byte $40
          .byte $22
          .byte $43
          .byte $44
          .byte $40
          .byte $06
          .byte $40
          .byte $08
          .byte $40
          .byte $02
          .byte $43
          .byte $44
          .byte $40
          .byte $06
          .byte $40
          .byte $08
          .byte $40
          .byte $02
          .byte $43
          .byte $44
          .byte $40
          .byte $06
          .byte $40
          .byte $08
          .byte $00
          .byte $22
          .byte $44
          .byte $44
          .byte $40
          .byte $69
          .byte $44
          .byte $00
          .byte $11
          .byte $22
          .byte $44
          .byte $44
          .byte $40
          .byte $69
          .byte $44
          .byte $87
          .byte $10
          .byte $22
          .byte $44
          .byte $44
          .byte $40
          .byte $06
          .byte $40
          .byte $08
          .byte $10
          .byte $22
          .byte $44
          .byte $44
          .byte $40
          .byte $06
          .byte $40
          .byte $08
          .byte $A2
          .byte $14
          .byte $56
          .byte $78
;
; Message
;
MESSG
        .byte CR
        .text "PASS 1"
        .text @
        .byte CR
        .text "PASS 2"
        .byte CR
        .text "PRINT? "
        .text @
        .byte CR
        .text "SAVE ID= "
        .text @
        .text "    MICRO-WARE ASSEMBLER 65XX-1.0 PAGE "
        .text @
        .byte CR
        .text "    SYMBOL TABLE "
        .text @
        .byte CR
        .text "ID="
        .text @
        .byte CR
        .text "NEW?"
        .text @
        .byte CR
        .text "CLEAR"
        .text @
;
;        .byte CR
;        .text "PASS 1"
;        .text @
;        .byte CR
;        .text "PASS 2"
;        .byte CR
;        .text "PRINT? "
;        .text @
;        .byte CR
;        .text "SAVE ID= "
;        .text @
;        .text "    MICRO-WARE ASSEMBLER 65XX-1.0 PAGE "
;        .text @
;        .byte CR
;        .text "    SYMBOL TABLE "
;        .text @
;        .byte CR
;        .text "ID="
;        .text @
;        .byte CR
;        .text "NEW?"
;        .text @
;        .byte CR
;        .text "CLEAR"
;        .text @
;
; ****************************
; * MICRO-ADE Disassembler*(C)*  
; ****************************
;           
DSMBLR    JSR CRLF              ; CRLF
          LDA PCHI
          LDX PCLO
          JSR HEXAX             ; print PC
;
          LDX #$00
          LDA (PCLO,X)          ; get OP
          TAY
          LSR A
          BCC EVEN
          LSR A
          BCS ERRZ
          CMP #$22              ; hash mode
          BEQ ERRZ
          AND #$07              ; table index
          ORA #$80
EVEN      LSR A
          TAX
          LDA MODE,X
;
          BCS RIGHT
          LSR A
          LSR A
          LSR A
          LSR A                 ; decode addres mode index
RIGHT     AND #$0F
          BNE GETFMT
ERRZ      LDA #$04
          LDY #$80              ; (error)
GETFMT    TAX
          STA INDX
;
          STY OP
          DEX
          JSR TTWO
          JSR ADJLTH            ; obtain length
;
          TYA
          AND #$8F
          TAX
          TYA
          LDY #$00              ; format mnemonic table index
          STY AX
          LSR A
          ROL AX
          LSR A
          ROL AX
          LSR A
          ROL AX
          BEQ ZUT
          LSR A
          LDY #$20
          CPX #$8A
          BEQ Y
          LSR A
          LSR ARGIN
          LDX ARGIN
          LDY TABL,X
Y         AND #$07

ZUT       STY AX
          ORA AX
          STA AX
          LDY #$00
          JSR OUTSP
          
LOOPD     LDA (PCLO),Y
          JSR HEXOUT            ; print opcode
          LDX #$01
LPX       JSR OUTSPX
          CPY LENGTH
          INY
          BCC LOOPD
;          
          LDX #03 
          CPY #04      
          BCC LPX
          LDX PCLO              ; print label
          LDY PCHI
          LDA #$06
          STA SPCNT
          JSR PSYMXY
;
          LDY AX
          LDA MNL,Y
          STA AX
          LSR A                 ; print mnemonic
          LSR A
          JSR ZUTP
          LDA MNR,Y
          ROL A
          ROL AX
          ROL A
          ROL AX
          ROL A
          ROL AX
          LDA AX
          JSR ZUTP
          LDA MNR,Y
          JSR ZUTP
;
          LDX INDX              ; print address mode
          LDA TABLL - 1,X
          JSR OUTCH
          LDA TABLR - 1,X
          JSR OUTCH
          JSR OUTSP
          LDX LENGTH
          DEX
          BMI NONE
          BNE TWZ
;
          LDY #$01
          LDA OP
          AND #$1F
          CMP #$10
          BNE ZTREL
;
          LDA (PCLO),Y
          PHA
          SEC
          ADC #$01
          ADC PCLO
          TAX
          LDA PCHI
          ADC #$00
          TAY
          PLA
          BPL ZNE
          DEY
          JMP ZNE
;
ZTREL     LDA (PCLO),Y
          TAX
          DEY
          BEQ ZNE
;
TWZ       LDY #$01
          LDA (PCLO),Y
          TAX
          INY
          LDA (PCLO),Y
          TAY
;
ZNE       LDA #$00
          STA SPCNT             ; print argument
          JSR PSYMXY
NONE      JSR PCADJ
          RTS
;
; routine to print X spaces
;
OUTSPX    JSR OUTSP
          DEX
          BNE OUTSPX
          RTS
;
ZUTP      AND #$1F
          ORA #$40
          JMP OUTCH
;
; Print symbol of $XY
;
PSYMXY    STX XCLO
          STY XCHI
          JSR SETSY
;
SRCH      LDY #$04
          LDA (SLO),Y
          CMP XCLO
          BNE INC               ; search table
          INY
          LDA (SLO),Y
          CMP $2A
          BNE INC
          JMP DPSYM             ; print table
;
INC       JSR DADS
          BCC SRCH
;
          LDY SPCNT
          BNE BLNK              ; no symbol in table, use $XY
          LDA #'$'
          JSR OUTCH
          LDA XCHI
          LDX XCLO
          JSR HEXAX
          JSR OUTSP
;
BLNK      JSR OUTSP
          DEY                   ; fill blanks
          BPL BLNK
          RTS
;
; Filler? Contents of memory at merge objects?
; Unused but present in dump, kept here to let the binary be byte correct
;         
          .byte $26
          .byte $FF
          .BYTE $E3 ;11100011
          .BYTE $FF ;11111111
          .BYTE $EB ;11101011
;
; *****************************
; * Input and Output Routines *
; * For the MCRO-ADE system   *
; *****************************
; IO org $2DC5 77.06.24
;         
          
NOUT      LDA NHI               ; Get HI N
          LDX NLO               ; get Lo N .. print them
;
; sub to print 2 hex bytes
; first byte in A, second byte in X
;
HEXAX     JSR HEXOUT            ; print accumulator
          TXA                   ; get byte in X .. print it
;
; subroutine to print 2 hex bytes
;
HEXOUT    PHA                   ; save input
          LSR A                 ; get upper nybble
          LSR A
          LSR A
          LSR A
          JSR HEX               ; print upper nybble
          PLA                   ; get input back
          AND #$0F              ; get lower nybble
; 
; subroutine to print 1 hex byte in A
;          
HEX       CMP #$0A              ; letter or number
          CLC                   ; calculate ASCII   
          BMI HEXA              ; if it is a number
          ADC #$07              ; add 7 to letter
HEXA      ADC #$30              ; add 30 to both
          BNE OUTCH             ; then print it
;
; Subroutine to print a backspace
; if your terminal can't change to 5F to another character 
; another character to indicate deletes
;
BACKSP    LDA #BSP              ; backspace character
          BNE OUTCH             ; print it
;
; subroutine to print carriage return and linefeed
;
CRLF      LDA #CR               ; get CR character
          BNE OUTCH             ; and print it
;
; subroutine to print 2 hex bytes followed by a space 
;
HEXSP     JSR HEXAX             ; print 2 hex bytes
;
; subroutine to print a space
;
OUTSP     LDA #$20              ; load space in A
;
; subroutine to print an ASCII character in A
;
OUTCH     STY YTMP              ; hide Y 
          STX XTMP              ; and X
          CMP #CR               ; is this a CR?
          BNE NOCR              ; skip LF
;
          LDX PMODE             ; check pagemode flag
          BNE NOPG              ; skip if not on
          INC COUNTL            ; add 1 to lines printed
          BNE NOPG              ; skip if not end of screen
;
          JSR INCH     
          CMP #ESC              ; was escape key entered?
          BNE ON                ; if not continue in page mode
          LDA #$FF
          STA PMODE             ; turn off pagemode
;
ON        LDX #$F0
          STX COUNTL            ; reset linecounter to -16
;
NOPG      LDA #LF               ; print linefeed      
          JSR NOCR              ; (remove if your termianl has auutoLF)
; 
          LDA #CR               ; this was CR remember              ; 
NOCR      JSR OUTPUT            ; so print it
;
; routine to test for break during I/O
;
BRKTST    BIT SAD               ; test input port of 6530 
          BPL BREAK             ; if PB7 is 0
          LDX XTMP              ; seek hidden X and Y
          LDY YTMP
          RTS
;          
BREAK     BIT SAD               ; wait until key is released
          BPL BREAK
          JMP RESTRT            ; then go to editor
;
; routine to input an ASCII character
; returns it in accumulator 
;          
INCH      STX XTMP              ; hide X and Y
          STY YTMP
          JSR INPUT             ; call user input routine
          AND #$7F              ; strip parity bit
          CMP #CR               ; was input a Return?
          BNE NOCRIN            ; if not its OK
          LDA #LF               ; print LF with CR input
          JSR OUTPUT            ; skip this of AUTO LF on terminal    
          LDA #CR               ; replace CR for RTS
NOCRIN    BNE BRKTST            ; return via breaktest
;
; subroutine to fill buffer from keyboard input
;
BUFIN     LDY #$00              ; reset buffer counter
INB       JSR INCH              ; get character input
          CMP #DEL              ; was it a DELETE
          BNE ONIN              ; if not carry on
          JSR BACKSP            ; print a backspace
          DEY                   ; backup in buffer
          BPL INB               ; and get it right this time
          BRK                   ; error, backed up too far! 
;
ONIN      CMP #BSL              ; backslash will delete whole line
          BNE OKB
          JSR CRLF              ; print CRLF
          JMP BUFIN             ; and start all over
;
OKB       CMP #CTRLE            ; was it CTRL-E?
          BEQ OVR               ; yes goto OVR function
          STA BUFFER,Y           ; just an ordinary character to RETPC
          CMP #CR               ; was this the end?
          BEQ ENDBU             ; yes so get out of here
          INY                   ; increment pointer
          CPY #$3A              ; allow only 58 cars + 6 prompt = 64
          BMI INB               ; still some room for more
          LDA #CR               ; force CR to end of line
          BNE OKB               ; print it and put in buffer
ENDBU     RTS                   ; all done
;
OVR       JSR BACKSP            ; cancel the CTL character (this may be 
                                ; necessary on some terminals)
OVX       LDA BUFFER,Y           ; get character in buffer
          CMP #CR               ; is it the end?
          BEQ INB               ; if so get new addition
          JSR OUTCH             ; show what it is
          INY                   ; on to next character
          CPY #$3A              ; but don't get carried ways! BUF max 64
          BNE OVX               ; keep going
          BEQ INB               ; let him fix it up
;
; subroutine to print the buffer
;
PRBUF     LDY #$00              ; reset the pointer
PRNTB     LDA BUFFER,Y          ; get a character
          PHA                   ; hide it temporarily
          JSR OUTCH             ; print it
          PLA                   ; seek it back
          INY                   ; point to next character
          CMP #CR               ; was that the end of the buffer
          BNE PRNTB             ; no there must be more
          RTS                   ; yes so return
;
;      ************************
; **** User specified addresses *****
;      ************************
;
PACKT     JMP $1A00             ; KIM routine to pack ASCI in byte
READ      JMP CREAD             ; Cassette read routine
WRITE     JMP CWRITE            ; Cassette write routine
INPUT     JMP $1E5A             ; keybaord input routine
OUTPUT    JMP $1EA0             ; printer output routine
;
; definition of SOURCF location
;
SOURCM  .byte $35               ; SOURCF -1
SOURCE  .byte $36               ; SOURCF area of memory starts here
SOURCF  .byte $40               ; and ends just below here
;
; Definition of Symbol table location
;
SYMBOL  .byte $30           ; Symbol table starts here
ENDSYM
SYMF    .byte $36           ; and just below here
;
; Definition of object location
;
OBJECT .byte $02            ; The object will be assembled to here
;
; ****************************************
; * KIM Cassette Read and Write routines *
; ****************************************
ERID      JSR HEXOUT
          JSR OUTSP
;
; Cassette Read Subroutine
;
; Very similar to the read routine in the KIM-1          
; The LED display of incoming data adapted
; from VUtape, a program by Jim Butterfield
; from the KIM-1 User Notes.
; 
CREAD     LDA SBD2              ; turn on cassette #1 by
          AND #$FB              ; changing bit 2 to 0 in PIA port B
          STA SBD2
          LDA #$7F              ; turn on the KIM LED display             
          STA PADD              ; by setting the DD register                
;
          CLD                   ; just to be sure
;
          LDA #$8D              ; set up VEB to save data
          STA VEB
          JSR INTVEB            ; in KIM ROM
;
          LDA #$13              ; turn on input port from cassette
          STA SBD               ; 
;
SYNC      JSR RDBIT             ; start reading a bit at a time
;
          LSR TMP               ; shift into TMP
          ORA TMP               ; and save it
          STA TMP               ; and save it
          STA SAD               ; place it on the LED
;
TST       CMP #$16              ; is it a sync character?
          BNE SYNC              ; if not keep trying
;
          JSR RDCHT             ; in sync, read a character
          STA SAD               ; display it on LED
          CMP #$2A              ; is it the start of data?
          BNE TST               ; if not loop again
;
          JSR RDBYT             ; read the tape ID
          CMP ID                ; is this the right tape ID?
          BNE ERID              ; print it if wrong
;
          JSR RDBYT             ; read the start address
          JSR CHKT              ; include it in checksum
          STA VEB + 1           ; and save it in VEB
          JSR RDBYT             ; Read the hi part of the address
          JSR CHKT              ; include in checksum
          STA VEB + 2           ; set it up in VEB
;
LOADIT    LDX #$02              ; start to laod data as 
READIT    JSR RDCHT             ; ASCII characters
          CMP #$2F              ; end of data symbol
          BEQ ENDRD             ; so wind it up
;
          JSR PACKT             ; pack the ASCII into hex
          BNE SYNC              ; error in character read 
          DEX                   ; count to two
          BNE READIT            ; read second half
;
          JSR CHKT              ; include in checksum
          JSR VEB               ; RETPC via VEB
          JSR INCVEB            ; increment RETPC address
          JMP LOADIT            ; and load next byte
;
ENDRD     JSR RDBYT             ; read checksum from tape
          CMP CHKL              ; compare to calculated
          BNE SYNC              ; and start over if wrong
          JSR RDBYT             ; get second half of sum
          CMP CHKH              ; and do the same with it
          BNE SYNC

          LDA SBD2              ; turn off cassette bit 2
          ORA #$04              ; by setting bit 2 of PIA port
          STA SBD2
          JMP INIT              ; return via INIT (reset all ports)
;
; *** KIM Cassette Write Subroutine ***
;
; adapted from Supertape by Jim Butterfield
; as published by in KIM-1 USer Notes (V T, N 2)
;          
CWRITE    LDA SBD2              ; turn on cassette #2
          AND #$F7              ; by setting bit 3 to 0
          STA SBD2              ; in PIA port B
;
          LDA #$AD              ; setup VEB for save
          STA VEB               ;
          JSR INTVEB            ; 
;
          LDA #$27              ; set flag for SBD later
          STA GANG              ; 
;
          LDA #$BF              ; turn on output to cassette
          STA PBDD              ; 
;
          LDX #$F0              ; send 240 sync pulses (optimal recorder start/stop time)
;
          LDA #$16              ; sync character
          JSR HIC               ; output X times
;
          LDA #$2A              ; send start of data character
          JSR OUTCHT            ; 
;
          LDA ID                ; get ID
          JSR OUTBT             ; and send as byte
;
          LDA SALX              ; send execution address
          JSR OUTBTC            ; with checksum calculation
          LDA SAHX              ; hi part too
          JSR OUTBTC
;
DUMPTA    JSR VEB               ; get a byte of memory
          JSR OUTBTC            ; send and checksum it
          JSR INCVEB            ; point to next byte
          LDA VEB + 1           ; check for end  against EAL
          CMP EAL
          LDA VEB + 2           ; and EAH
          SBC EAH
          BCC DUMPTA            ; again if not end
;
          LDA #$2F              ; end end of data character
          JSR OUTCHT            ; as char
;
          LDA CHKL              ; send checksum lo and hi
          JSR OUTBT
          LDA CHKH
          JSR OUTBT
          LDX #$02              ; send 2 EOT characters
          LDA #$04
          JSR HIC
;
          LDA SBD2              ; turn off cassette 
          ORA #$08              ; by setting bit 3
          STA SBD2              ; of the control port
          JMP INIT              ; reset all ports
;
; subroutine to send X characters to tape
;  
HIC       STX TIC         
HICA      PHA
          JSR OUTCHT            ; send the character
          PLA                   ; and get it back
          DEC TIC               ; to send again
          BNE HICA              ; until count = 0
          RTS
; 
OUTBTC    JSR CHKT
; 
; subroutine to send byte as two ASCII chars
;
OUTBT     PHA                   ; save byte
          LSR A                 ; get upper nybble
          LSR A
          LSR A
          LSR A
          JSR HEXT              ; and send it
          PLA                   ; return byte and sent lower nybble
;
;  subroutine to send one HEX char as ASCII
;          
HEXT      AND #$0F              ; cleanup data (lower nybble)
          CMP #$0A              ; change to ASCII
          CLC                   
          BMI HEXAT
          ADC #$07              ; 37 to A..F
HEXAT     ADC #$30              ; 30 to 0..9
;
; subroutine to send one 8 bit byte
;
OUTCHT    LDY #$08              ; eight bit count
          STY COUNT             
TRY       LDY #$02              ; start at 3600 Hz
          STY TRIB              ;
ZON       LDX NPUL,Y            ; number if half cycles
          PHA                   ; save char
ZONA      BIT TIMER             ; wait for end of cycle            
          BPL ZONA              ; in tight loop
;
          LDA $TIMG,Y           ; setup timer
          STA TIMERD            ; change state of output port
;
          LDA GANG              ; 
          EOR #$80
          STA SBD
;
          STA GANG              ; save state
          DEX                   ; done all cycles?
          BNE ZONA
;
          PLA                   ; retrieve byte
          DEC TRIB              ; one more gone
          BEQ SETZ              ; the last one too
          BMI ROUT              ; even the last one went
;
          LSR A                 ; another bit to the carry
          BCC ZON               ; if it is not set
SETZ      LDY #$00              ; switch to 2400 Hz
          BEQ ZON               ; always
;
ROUT      DEC COUNT             ; one bit sent
          BNE TRY               ; but more to go
          RTS                   ; All over, go home
;
; Timing table
;          
NPUL      .BYTE $02 ;00000010
TIMG      .BYTE $C3 ;11000011
          .BYTE $03 ;00000011
          .BYTE $7E
; If your recorder cannot handle this speed
; you can slow down by changing NPUL and NPUL2
; to one of the following        03       04
;                                06       09
; This is the KIM ROM speed      0C       12
          .END
